import logging
import pathlib
import pickle
import random

import discord

logger = logging.getLogger(__name__)


class Wordlist:
    def __init__(
        self,
        bot: discord.Bot,
        guild_ids: list[int] = [],
        path: pathlib.Path = pathlib.Path("wordlist.pickle"),
    ):
        self.path = path
        if self.path.exists():
            with self.path.open("rb") as w_file:
                self.word_list: list[str] = pickle.load(w_file)
        self.bot = bot
        self.guild_ids = guild_ids

    def init_events(self) -> None:
        self.bot.add_application_command(
            discord.SlashCommand(
                self.bibl,
                name="bibl",
                description="Ajouter une phrase",
                guild_ids=self.guild_ids,
            )
        )
        self.bot.add_application_command(
            discord.SlashCommand(
                self.tabl,
                name="tabl",
                description="Lister les phrases",
                guild_ids=self.guild_ids,
            )
        )
        self.bot.add_application_command(
            discord.SlashCommand(
                self.enle,
                name="enle",
                description="Enlever une phrase",
                guild_ids=self.guild_ids,
            )
        )

    def random(self) -> str:
        return random.choice(self.word_list)

    def save(self) -> None:
        logger.info("Saving updated wordlist")
        with open(self.path, "wb") as w_file:
            pickle.dump(self.word_list, w_file)

    async def bibl(self, ctx: discord.ApplicationContext, phrase: str) -> None:
        logger.info(f"BIBL {ctx.author} {phrase}")
        self.word_list.append(phrase)
        embed = discord.Embed(
            title="BIBL", description=phrase, color=discord.Colour.green()
        )
        await ctx.respond(embed=embed)
        self.save()
        logger.info("FIN BIBL")

    async def tabl(self, ctx: discord.ApplicationContext) -> None:
        logger.info(f"TABL {ctx.author}")
        embed = discord.Embed(
            title="TABL",
            description="\n".join(self.word_list),
            color=discord.Colour.green(),
        )
        await ctx.respond(embed=embed, ephemeral=True, delete_after=30)

    async def enle(self, ctx: discord.ApplicationContext, phrase: str) -> None:
        logger.info(f"ENLE {ctx.author} {phrase}")
        try:
            self.word_list.remove(phrase)
        except ValueError:
            embed = discord.Embed(
                title="ERRE ENLE", description=phrase, color=discord.Colour.red()
            )
            await ctx.respond(embed=embed)
            logger.info("ERRE ENLE")
        else:
            embed = discord.Embed(
                title="ENLE", description=f"~~{phrase}~~", color=discord.Colour.green()
            )
            await ctx.respond(embed=embed, ephemeral=True, delete_after=30)
            self.save()
            logger.info("FIN ENLE")
